@@ -29,36 +29,31 @@ from multiprocessing import Pool, cpu_count
 from functools import partial
 import mpmath as mp
 import pandas as pd
-from patsy import dmatrix
 
 #import unittest
 
 
-def model_matrix(batch):
+def model_matrix(info, intercept=True, drop_first=True):
     """Creates the model_matrix from batch list
 
     Arguments:
-        batch {list} -- batch list from main arguments
+        info {list} -- list info with batch or covariates data
+        intercept {bool} -- boolean for intercept in model matrix
 
     Returns:
         matrix -- model matrix generate from batch list
     """
-    # list_elts_batch = list(
-    #     set(batch))
-    list_elts_batch = sorted(list(set(batch)))
-    n = len(batch)
-    n_batch = len(list_elts_batch)
-    dict_batch = {}
-
-    mat = [[0] * n for _ in range(n_batch)]
-
-    for i in range(n_batch):
-        dict_batch[list_elts_batch[i]] = i
-
-    for i in range(n):
-        mat[dict_batch[batch[i]]][i] = 1
-
-    return mat
+    if not isinstance(info[0],list) :
+        info = [info]
+    else:
+        info = info
+    info_dict = {}
+    for i in range(len(info)):
+        info_dict[f"col{str(i)}"] = list(map(str,info[i]))
+    df = pd.get_dummies(pd.DataFrame(info_dict), drop_first=drop_first, dtype=float)
+    if intercept:
+        df["intercept"] = 1.0
+    return df.to_numpy()
 
 
 def all_1(list_of_elements):
@@ -72,6 +67,7 @@ def all_1(list_of_elements):
     """
     return((list_of_elements == 1).all())
 
+
 # aprior and bprior are useful to compute "hyper-prior values"
 # -> prior parameters used to estimate the prior gamma distribution for multiplicative batch effect
 # aprior - calculates empirical hyper-prior values
@@ -307,7 +303,7 @@ def define_batchmod(batch):
     Returns:
         batchmod {matrix} -- model matrix for batches
     """
-    batchmod = model_matrix(batch)
+    batchmod = model_matrix(list(batch), intercept=False, drop_first=False)
     return(batchmod)
 
 
@@ -323,7 +319,7 @@ def check_ref_batch(ref_batch, batch, batchmod):
         ref {int list} -- the corresponding positions of the reference batch in the batch list
         batchmod {matrix} -- updated model matrix related to batches, with reference
     """
-    if ref_batch:
+    if ref_batch is not None:
         if ref_batch not in batch:
             print("Reference level ref.batch must be one of the levels of batch.")
             exit(0)
@@ -359,7 +355,7 @@ def treat_batches(batch):
     n_batches = list(map(len, batches))
     if 1 in n_batches:
         #mean_only = True  # no variance if only one sample in a batch - mean_only has to be used
-        print("One batch has only one sample, try setting mean_only=True.")
+        print("\nOne batch has only one sample, try setting mean_only=True.\n")
     n_array = sum(n_batches)
     return(n_batch, batches, n_batches, n_array)
 
@@ -377,13 +373,12 @@ def treat_covariates(batchmod, mod, ref, n_batch):
         check {bool list} -- a list characterising all covariates
         design {matrix} -- model matrix for all covariates, including batch
     """
-    if mod == []:
-        design = dmatrix("~-1 + batchmod")
-    else:
-        mod_matrix = dmatrix("~C(mod)")
-        design = dmatrix("~-1 + batchmod + mod_matrix")
     # design matrix for sample conditions
-    design = np.asarray(design)
+    if mod == []:
+        design = batchmod
+    else:
+        mod_matrix = model_matrix(mod, intercept=True)
+        design = np.concatenate((batchmod, mod_matrix), axis=1)
     check = list(map(all_1, np.transpose(design)))
     if ref is not None:  # if ref
         check[ref] = False  # the reference in not considered as a covariate
@@ -423,7 +418,7 @@ def check_NAs(dat):
     # NAs = True in (np.isnan(dat))
     NAs = np.isnan(np.sum(dat))  # Check if NaN exists
     if NAs:
-        print("Found missing data values.")
+        print("Found missing data values. Please remove all missing values before proceeding with pyComBat.")
     return(NAs)
 
 
@@ -631,17 +626,17 @@ def pycombat(data, batch, mod=[], par_prior=True, prior_plots=False, mean_only=F
         batch {list} -- List of batch indexes. The batch list describes the batch for each sample. The batches list has as many elements as the number of columns in the expression matrix.
 
     Keyword Arguments:
-        mod {list} -- List of covariates indexes. The mod list describes the covariate for each sample. The mod list has as many elements as the number of columns in the expression matrix (default: {[]}).
+        mod {list} -- List (or list of lists) of covariate(s) indexes. The mod list describes the covariate(s) for each sample. Each mod list has as many elements as the number of columns in the expression matrix (default: {[]}).
 
-        par_prior {bool} -- False for non-parametric estimation of batch effects (default: {True})
+        par_prior {bool} -- False for non-parametric estimation of batch effects (default: {True}).
 
-        prior_plots {bool} -- True if requires to plot the priors (default: {False} -- Not implemented yet!)
+        prior_plots {bool} -- True if requires to plot the priors (default: {False} -- Not implemented yet!).
 
-        mean_only {bool} -- True iff just adjusting the means and not individual batch effects (default: {False})
+        mean_only {bool} -- True iff just adjusting the means and not individual batch effects (default: {False}).
 
-        ref_batch {int} -- reference batch selected (default: {None})
+        ref_batch {int} -- reference batch selected (default: {None}).
 
-        precision {float} -- level of precision for precision computing (default: {None})
+        precision {float} -- level of precision for precision computing (default: {None}).
 
     Returns:
         bayes_data_df -- The expression dataframe adjusted for batch effects.
@@ -653,23 +648,26 @@ def pycombat(data, batch, mod=[], par_prior=True, prior_plots=False, mean_only=F
 
     check_mean_only(mean_only)
 
-    batchmod = dmatrix("~-1 + C(batch)")
+    batchmod = define_batchmod(batch)
     ref, batchmod = check_ref_batch(ref_batch, batch, batchmod)
     n_batch, batches, n_batches, n_array = treat_batches(batch)
     design = treat_covariates(batchmod, mod, ref, n_batch)
     NAs = check_NAs(dat)
-    B_hat, grand_mean, var_pooled = calculate_mean_var(
-        design, batches, ref, dat, NAs, ref_batch, n_batches, n_batch, n_array)
-    stand_mean = calculate_stand_mean(
-        grand_mean, n_array, design, n_batch, B_hat)
-    s_data = standardise_data(dat, stand_mean, var_pooled, n_array)
-    gamma_star, delta_star, batch_design = fit_model(
-        design, n_batch, s_data, batches, mean_only, par_prior, precision, ref_batch, ref, NAs)
-    bayes_data = adjust_data(s_data, gamma_star, delta_star, batch_design,
-                             n_batches, var_pooled, stand_mean, n_array, ref_batch, ref, batches, dat)
-    
-    bayes_data_df = pd.DataFrame(bayes_data,
-                 columns = list_samples,
-                 index = list_genes)
+    if not(NAs):
+        B_hat, grand_mean, var_pooled = calculate_mean_var(
+            design, batches, ref, dat, NAs, ref_batch, n_batches, n_batch, n_array)
+        stand_mean = calculate_stand_mean(
+            grand_mean, n_array, design, n_batch, B_hat)
+        s_data = standardise_data(dat, stand_mean, var_pooled, n_array)
+        gamma_star, delta_star, batch_design = fit_model(
+            design, n_batch, s_data, batches, mean_only, par_prior, precision, ref_batch, ref, NAs)
+        bayes_data = adjust_data(s_data, gamma_star, delta_star, batch_design,
+                                n_batches, var_pooled, stand_mean, n_array, ref_batch, ref, batches, dat)
+        
+        bayes_data_df = pd.DataFrame(bayes_data,
+                    columns = list_samples,
+                    index = list_genes)
 
-    return(bayes_data_df)
\ No newline at end of file
+        return(bayes_data_df)
+    else:
+        raise ValueError("NaN value is not accepted")
\ No newline at end of file
